package com.winhc.bigdata.spark.jobs.dynamic

/**
 * @Author: XuJiakai
 * @Date: 2020/7/27 17:05
 * @Description:
 */
trait CompanyDynamicHandle {

  private val table_2_sub_info_type_map = Map("CompanyDynamicHandleTest" -> "MyTest")

  private val table_2_info_type = Map("CompanyDynamicHandleTest" -> "0")

  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return info_type
   *         rta_desc
   *         change_content
   *         change_time
   *         biz_id
   *         sub_info_type
   *         info_risk_level
   *         winhc_suggest
   */
  def handle(rowkey: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String]): (String, String, String, String, String, String, String, String) = {
    (get_info_type()
      , get_rta_desc(old_map, new_map)
      , get_change_content(old_map, new_map)
      , get_change_time(new_map)
      , get_biz_id(rowkey)
      , get_sub_info_type()
      , get_info_risk_level(old_map, new_map)
      , "被监控企业流动资金紧张，可能存在经营困难的情况。建议立即与被监控企业书面对账，适当催促其履行债务并持续监控。"
    )

  }


  /**
   * 来源表的变更类型，insert or update
   *
   * @return
   */
  def org_type(): Seq[String] = Seq("insert")


  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String

  /**
   * 信息类型，大类
   *
   * @return
   */
  def get_info_type(): String = table_2_info_type(getClass.getSimpleName)

  /**
   * 变更内容
   *
   * @param old_map
   * @param new_map
   * @return
   */
  def get_change_content(old_map: Map[String, String], new_map: Map[String, String]): String

  /**
   * 变更时间
   *
   * @param new_map
   * @return
   */
  def get_change_time(new_map: Map[String, String]): String

  /**
   * 业务id
   *
   * @param rowkey
   * @return
   */
  def get_biz_id(rowkey: String): String = rowkey.split("_")(1)

  /**
   * 子信息类型，小类
   *
   * @return
   */
  def get_sub_info_type(): String = table_2_sub_info_type_map(getClass.getSimpleName)

  /**
   * 风险等级
   *
   * @param old_map
   * @param new_map
   * @return
   */
  def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String

}
